// Copyright 2011 Julian Phillips.  All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package py

// #cgo CFLAGS: -Werror
// #cgo LDFLAGS: -lpython2.7
// #cgo pkg-config: libffi
//
// #include "utils.h"
//import "C"

import (
	"unsafe"
)

func Initialize() {
	_Py_Initialize()
}

func InitializeEx(initsigs bool) {
	if initsigs {
		_Py_InitializeEx(1)
	} else {
		_Py_InitializeEx(0)
	}
}

func Finalize() {
	_Py_Finalize()
}

func AddToPath(dir string) {
	p := _CString("path")
	defer _free(unsafe.Pointer(p))

	sys_path := _PySys_GetObject(p)
	if sys_path == nil {
		return
	}

	s := _CString(dir)
	defer _free(unsafe.Pointer(s))

	pDir := _PyString_FromString(s)
	if pDir == nil {
		return
	}

	_PyList_Append(sys_path, pDir)
}

func Main(args []string) int {
	argv := make([]*int8, len(args))

	for i, arg := range args {
		argv[i] = _CString(arg)
		defer _free(unsafe.Pointer(argv[i]))
	}

	return int(_Py_Main(int32(len(argv)), &argv[0]))
}

// EnterRecusiveCall marks a point where a recursive Go-level call is about to
// be performed.  It returns true if the recursive call is permitted, otherwise
// a Python exception is set and false is returned.  where is a string that will
// be appended to the RuntimeError set if the recursion limit has been exceeded
// (e.g. " in instance check").  This function needs to be called if the
// recursive function may not invoke Python code (which automatically tracks
// recursion depth).
func EnterRecursiveCall(where string) bool {
	s := _CString(where)
	defer _free(unsafe.Pointer(s))
	return _enterRecursive(s) == 0
}

// LeaveRecursiveCall must be called after a recursive call that was indicated
// by EnterRecursiveCall.
func LeaveRecursiveCall() {
	_leaveRecursive()
}
