// Copyright 2011 Julian Phillips.  All rights reserved.
// Use of this source code is governed by a BSD-style
// license that can be found in the LICENSE file.

package py

// #include "utils.h"
//import "C"

import "unsafe"

type String struct {
	AbstractObject
	o _PyStringObject
}

// StringType is the Type object that represents the String type.
var StringType = (*Type)(unsafe.Pointer(&*_PyString_Type))

func stringCheck(obj Object) bool {
	return _stringCheck(c(obj)) != 0
}

func newString(obj *_PyObject) *String {
	return (*String)(unsafe.Pointer(obj))
}

func NewString(s string) (*String, error) {
	cs := _CString(s)
	defer _free(unsafe.Pointer(cs))
	ret := _PyString_FromString(cs)
	if ret == nil {
		return nil, exception()
	}
	return newString(ret), nil
}

func (s *String) String() string {
	if s == nil {
		return "<nil>"
	}
	ret := _PyString_AsString(c(s))
	if ret == nil {
		panic(exception())
	}
	return _GoString(ret)
}

func (s *String) Format(args *Tuple) (*String, error) {
	ret := _PyString_Format(c(s), c(args))
	if ret == nil {
		return nil, exception()
	}
	return newString(ret), nil
}
